<?php
/*
Plugin Name: Falang for YOOtheme Lite
Description: Manage translation for YOOtteme with Falang
Author: Faboba
Author URI: https://www.faboba.com
Version: 1.18
*/

use YOOTHeme\Application;
use function YOOtheme\app;
use YOOtheme\Builder\Wordpress\Source\Helper;
use YOOtheme\Config;


Class FalangYOOTHeme {

    private $elements = [];
    private $elements_pro = []; //list of elements in pro version to display lite message
    private $default_language; //default language (Falang or QTranslate)
    private $language_list; //list of active language (Falang or QTranslate)
    private $current_language; //list of active language (Falang or QTranslate)

    private $admin_notices;

    /**
     * @var static
     */
    protected static $instance;


    function __construct() {
        define( 'FALANG_YOOTHEME_FILE',  __FILE__ ); // plugin name as known by WP
        define( 'FALANG_YOOTHEME_LITE_DIR', dirname( __FILE__ ) ); // plugin name as known by WP
        define( 'FALANG_YOOTHEME_LITE_PLUGIN_BASE', plugin_basename( FALANG_YOOTHEME_FILE ) );

        //Falang must be active.
        include_once(ABSPATH.'wp-admin/includes/plugin.php');
        if ( !is_plugin_active( 'falang/falang.php' ) ) {
            add_action( 'admin_notices', array($this,'falang_fail_load' ));
            return;
        }

        //TODO use autoload
        require_once FALANG_YOOTHEME_LITE_DIR.'/admin/admin-notices.php';

        //load all widgets
        add_action('plugins_loaded', array( $this,'init_elemets'));

        add_action('init', array( $this, 'init' ) );

    }

    /**
     * Gets global falangyootheme.
     *
     * @param null|mixed $cache
     *
     * @return static
     */
    public static function getInstance($cache = null)
    {
        return static::$instance ?: static::$instance = new static();
    }


    /**
     * @from 1.0
     * @update 1.4 add falang_is_supported_builder
     * @update 1/5 add yootheme_source_post_teaser filter
     *             add yootheme_source_post_content filter
     */
    function init() {
        // Check if YOOtheme Pro is loaded
        if (!class_exists(Application::class, false)) {
            return;
        }

        // Load a single module from the same directory
        $app = Application::getInstance();
        $app->load(__DIR__ . '/bootstrap.php');

        //register style
        wp_register_style ( 'falang-for-yootheme-lite', plugins_url ( 'admin/css/style.css', __FILE__ ) );
        wp_enqueue_style('falang-for-yootheme-lite');

        //load admin notice
        $this->admin_notices = new Falang\Yootheme\Admin\Admin_Notices();

        //remove content translation for Yootheme page
        add_filter( 'falang_is_supported_builder', [ $this, 'falang_is_supported_builder' ], 10, 2  );
        add_filter( 'yootheme_source_post_teaser', [ $this, 'yootheme_source_post_teaser' ], 10, 3  );
        add_filter( 'yootheme_source_post_content', [ $this, 'yootheme_source_post_content' ], 10, 3  );

    }

    private function getParamsIndexOfField($params,$key,$name){
        foreach ($params as $idx => $param){
            if ($param[$key] == $name ){
                return $idx;
            }
        }
        return -1;
    }

    /* Init json supported elemnets for translation
     *
     * @from 1.0
     * @update 1.6 remove forreskirby hd progress elements
     *             add pro elemenst lit for message
     *
     * */
    function init_elemets() {
        $falangYootheme = FalangYOOTHeme::getInstance();

        $falangYootheme->loadConfigElements(FALANG_YOOTHEME_LITE_DIR . '/elements/*.json');

        if ( !$this->is_falang_for_yootheme_pro_activated()) {
            $this->loadProMessageElements(FALANG_YOOTHEME_LITE_DIR . '/pro/*.json');
        }
    }

    public function loadConfigElements($path){
        $files = glob( $path );
        foreach ($files as $file){
            if(is_file($file)){
                $element = $this->loadJsonFile($file);
                if (isset($element['name'])){
                    $this->elements[$element['name']] = $element;
                }
            }
        }
    }

    /*
     * load pro element to display message on lite version
     *
     * @from 1.6
     * */
    public function loadProMessageElements($path){
        //load each element supported for translation
        $files = glob($path);
        foreach ($files as $file) {
            if (is_file($file)) {
                $element = $this->loadJsonFile($file);
                if (is_array($element)){
                    $this->elements_pro = array_merge($this->elements_pro,$element);
                }
            }
        }
    }


    public function getElements(){
        if (empty($this->elements)){
            $this->init_elemets();
        }
        return $this->elements;
    }

    /*
     * Get the list of pro elements use to display message
     * @from 1.6
     * */
    public function getProElements(){
        if (empty($this->elements_pro)){
            $this->init_elemets();
        }
        return $this->elements_pro;
    }


    /**
     * get the internal name for the field ex content_fr_fr
     * @param $name
     * @param $language
     *
     * @return string
     * 	 */
    public function getTranslatedFieldName($name, $language)
    {
        return $name . '_' . str_replace('-','_',strtolower($language->locale));
    }

    /**
     * get the internal name for the field label ex Content fr-FR (Joomla)
     * @udpate 1.9 first letter uppercase
     *
     * @param $name
     * @param $language
     *
     * @return string
     * 	 */
    public function getTranslatedFieldLabel($name, $language)
    {
        if (!empty($name)) {
            return ucfirst($name) . ' ' . ($language->locale);
        } else {
            return ($language->locale);
        }
    }


    /**
     * Loads a bootstrap config.
     *
     * @param string $file
     * @param array  $parameters
     *
     * @return array
     */
    protected function loadJsonFile($file)
    {
        if (!$content = @file_get_contents($file)) {
            throw new RuntimeException("Unable to load file '{$file}'");
        }

        if (!is_array($value = @json_decode($content, true))) {
            throw new RuntimeException("Invalid JSON format in '{$file}'");
        }

        return $value;
    }

    function get_current_language(){
        if (!isset($this->current_language)){
            $this->current_language = Falang()->get_current_language();
        }
        return $this->current_language;
    }

    function get_default_language() {
        if (!isset($this->default_language)){
            $this->default_language = Falang()->get_model()->get_default_language();
        }
        return $this->default_language;
    }

    /*
     * the arg are cached the first time
     * TODO make an array with args in prefix => not sure it's usefull for this small plugin
     * */
    function get_languages_list( $args = array() ) {
        if (empty($this->language_list)){
            $this->language_list = Falang()->get_model()->get_languages_list( $args );
        }
        return $this->language_list;
    }

    /**
     * Show in WP Dashboard notice about the plugin Falang is not activated.
     *
     * @since 1.1
     *
     * @return void
     */
    public function falang_fail_load() {
        $screen = get_current_screen();
        if ( isset( $screen->parent_file ) && 'plugins.php' === $screen->parent_file && 'update' === $screen->id ) {
            return;
        }

        $plugin = 'falang/falang.php';

        if ( $this->is_falang_installed() ) {
            if ( ! current_user_can( 'activate_plugins' ) ) {
                return;
            }

            $activation_url = wp_nonce_url( 'plugins.php?action=activate&amp;plugin=' . $plugin . '&amp;plugin_status=all&amp;paged=1&amp;s', 'activate-plugin_' . $plugin );

            $message = '<p>' . __( 'Falang for Yootheme Lite is not working because you need to activate the Falang plugin.', 'falang-for-yootheme' ) . '</p>';
            $message .= '<p>' . sprintf( '<a href="%s" class="button-primary">%s</a>', $activation_url, __( 'Activate Falang Now', 'falang-for-yootheme' ) ) . '</p>';
        } else {
            if ( ! current_user_can( 'install_plugins' ) ) {
                return;
            }

            $install_url = wp_nonce_url( self_admin_url( 'update.php?action=install-plugin&plugin=falang' ), 'falang-for-yootheme' );

            $message = '<p>' . __( 'Falang for Yootheme Lite is not working because you need to install the Falang plugin.', 'falang-for-yootheme' ) . '</p>';
            $message .= '<p>' . sprintf( '<a href="%s" class="button-primary">%s</a>', $install_url, __( 'Install Falang Now', 'falang-for-yootheme' ) ) . '</p>';
        }

        echo '<div class="error"><p>' . $message . '</p></div>';
    }

    private function is_falang_installed() {
        $file_path = 'falang/falang.php';
        $installed_plugins = get_plugins();

        return isset( $installed_plugins[ $file_path ] );
    }
    /**
     * @since 1.4 add builder name use to
     * @return false if not supported , YOOTheme if supported (use for display)
     */
    public function falang_is_supported_builder($return,$post){
            if ( 'yootheme' == get_template()) {
                if ($post and $post->builder = preg_match('/<!--\s?{/', $post->post_content)) {
                    $return = 'YOOTheme';
                }
            }
        return $return;
    }

    /**
     * @from 1.5 translate dynamic teaser
     * @return string
     */
    public function yootheme_source_post_teaser($teaser, $post, $args){
        $falang_post = new \Falang\Core\Post($post->ID);
        if (!Falang()->is_default() && $falang_post->is_post_type_translatable($post->post_type)) {
            $post = get_post($post->ID);
            $content = $falang_post->translate_post_field($post, 'post_content', Falang()->get_current_language(),$post->content);

            $extended = get_extended($content);
            $teaser = $extended['main'];

            // Having multiple `<!-- wp:more -->` blocks confuses the parse_blocks function
            if (has_blocks($teaser)) {
                $teaser = do_blocks($teaser);
            }

        }
        return $teaser;
    }

    /**
     * @from 1.5 translate dynamic content
     * @return string
     */
    public function yootheme_source_post_content($content, $post, $args){

        global $page, $numpages, $multipage;

        $args += ['show_intro_text' => true];

        $falang_post = new \Falang\Core\Post($post->ID);
        if (!Falang()->is_default() && $falang_post->is_post_type_translatable($post->post_type)) {
            $post = get_post($post->ID);
            $content = $falang_post->translate_post_field($post, 'post_content', Falang()->get_current_language(),$content);

            // Hint: this returns different results depending on the current view (archive vs. single page)
            $content = str_replace("<span id=\"more-{$post->ID}\"></span>", '', $content);

            if ($multipage && Helper::isPageSource($post)) {
                $title = sprintf(__('Page %s of %s', 'yootheme'), $page, $numpages);
                $content =
                    '<p class="uk-text-meta tm-page-break' .
                    ($page == '1' ? ' tm-page-break-first-page' : '') .
                    "\">{$title}</p>{$content}" .
                    link_pages();
            }

            if (!has_blocks($content) && !app(Config::class)('~theme.disable_wpautop')) {
                // trim leading whitespace, because ` </div>` results in `</p></div>
                $content = wpautop(preg_replace('/^\s+<\//m', '</', $content));
            }

        }
        return $content;
    }

    /**
     * @since 1.6 check if falang for elementor pro is active
     *
     */
    public function is_falang_for_yootheme_pro_activated(){
        return is_plugin_active( 'falang-for-yootheme/falang-for-yootheme.php' );
    }
}

//need to load only the instance
$FalangYOOTHeme = FalangYOOTHeme::getInstance();